"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const map_1 = __importDefault(require("./map"));
class OrderedMap extends Map {
    constructor(iterable = []) {
        // `super` will call `.set`, so no need to initialise `this.order`
        super(iterable);
    }
    get firstKey() {
        return this.order[0];
    }
    get firstValue() {
        return this.get(this.firstKey);
    }
    get lastKey() {
        return this.order[this.order.length - 1];
    }
    get lastValue() {
        return this.get(this.lastKey);
    }
    [Symbol.iterator]() {
        return this.entries();
    }
    entries() {
        return map_1.default(this.keys(), (k) => [k, this.get(k)]);
    }
    keys() {
        return this.order[Symbol.iterator]();
    }
    values() {
        return map_1.default(this.keys(), (k) => this.get(k));
    }
    clear() {
        super.clear();
        this.order = [];
    }
    delete(key) {
        const deleted = super.delete(key);
        if (deleted) {
            // TODO Improve performance
            this.order.splice(this.order.indexOf(key), 1);
        }
        return deleted;
    }
    forEach(callbackfn, thisArg) {
        super.forEach(callbackfn, thisArg);
    }
    get(key) {
        return super.get(key);
    }
    has(key) {
        return super.has(key);
    }
    set(key, value) {
        if (!this.order) {
            // For constructor (see `.order` comment)
            this.order = [];
        }
        if (!this.has(key)) {
            this.order.push(key);
        }
        return super.set(key, value);
    }
}
exports.default = OrderedMap;
