"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const every_1 = __importDefault(require("./every"));
function deepEquals(a, b) {
    if (a === b) {
        return true;
    }
    const typeA = typeof a;
    const typeB = typeof b;
    if (typeA != typeB) {
        return false;
    }
    // This handles direct comparisons between
    // undefined, null, strings, numbers, booleans, and functions
    // (although this function shouldn't be used to compare functions)
    if (a == null || typeA != "object") {
        // We already know a !== b.
        return false;
    }
    // This also works on arrays, but is inefficient. However, we shouldn't assume
    // that just because it has a "length" property it's an array-like.
    const keysA = new Set(Object.keys(a));
    const keysB = new Set(Object.keys(b));
    return (
    // WARNING: Object.keys([]) does not include "length" but [].hasOwnProperty("length") is true!
    // Therefore, we use set member testing instead of hasOwnProperty.
    every_1.default(keysA, (k) => keysB.has(k) && deepEquals(a[k], b[k])) &&
        every_1.default(keysB, (k) => keysA.has(k)));
}
exports.default = deepEquals;
